-- Tile type to use for base building
local basebuilding_tile_type = "stone-path";

-- Types of entities that cannot be placed outside of specific areas
local disallowed_types = {
	"storage-tank",
	"logistic-container",
	"roboport",
	"arithmetic-combinator",
	"decider-combinator",
	"constant-combinator",
	"boiler",
	"generator",
	"solar-panel",
	"accumulator",
	"furnace",
	"assembling-machine",
	"lab",
	"beacon",
	"radar",
	"rocket-silo"
}

-- We use "floor-layer" to restrict building placement
-- This items use floor-layer but should be placeable
local floor_items = {
	"straight-rail",
	"curved-rail",
	"transport-belt",
	"rail-signal",
	"rail-chain-signal"
}

local non_colliding_floor = {
	["straight-rail"] = true,
	["curved-rail"] = true
}

-- Special process function to disallow specific items
-- Substations are disabled while normal power poles are enabled
local special_disallowed_types = {
	["electric-pole"]=".*substation.*"
}

-- Map generation control
data:extend(
{
  {
    type = "noise-layer",
    name = "building-platform"
  },
  {
    type = "autoplace-control",
    name = "building-platform",
	richness = true, -- Should be false but it crashes the game
    order = "a-a"
  },
});

local function process_item(item)
	if not item then return end;
	if item.collision_mask then
		table.insert(item.collision_mask, "floor-layer");
	else
		item.collision_mask = { "item-layer", "object-layer", "player-layer", "water-tile", "floor-layer"}
	end;
end

local function unlock_floor(item)
	if not item then return end;
	if item.collision_mask then
		for i=1,#item.collision_mask do
			if item.collision_mask[i] == "floor-layer" then
				table.remove(item.collision_mask, i);
			end
		end
	else
		item.collision_mask = { "item-layer", "object-layer", "water-tile"}
		if non_colliding_floor[item.name] then
			table.insert(item.collision_mask, "not-colliding-with-itself")
		end
	end;
end

for i=1,#disallowed_types do
	local list = data.raw[disallowed_types[i]];
	for name,item in pairs(list) do
		process_item(item);
	end
end

for name,pattern in pairs(special_disallowed_types) do
	local list = data.raw[name];
	for name,item in pairs(list) do
		if string.match(name,pattern) == name then
			process_item(item);
		end;
	end
end

for i=1,#floor_items do
	local list = data.raw[floor_items[i]];
	for name,item in pairs(list) do
		unlock_floor(item);
	end
end

local tiles = data.raw["tile"];
local usetile = tiles[basebuilding_tile_type];

-- configuring building platform
usetile.minable = nil;
usetile.map_color = {r=100, g=100, b=100};
usetile.autoplace = {
	control = "building-platform",
	sharpness = 1,
	size_control_multiplier = 0.05,
	-- by increasing probability above default value we force this tile to be spawned first (don't mess with other tiles)
	max_probability=1.05,
	peaks = {
	-- regular placement
        {
          influence = 0.22,
          starting_area_weight_optimal = 0,
          starting_area_weight_range = 0,
          starting_area_weight_max_range = 2,
        },
        {
          influence = 0.75,
          noise_layer = "building-platform",
          noise_octaves_difference = -1.4,
          noise_persistence = 0.25,
          starting_area_weight_optimal = 0,
          starting_area_weight_range = 0,
          starting_area_weight_max_range = 2,
        },
	-- starting area placement
        {
          influence = 0.3,
          starting_area_weight_optimal = 1,
          starting_area_weight_range = 0,
          starting_area_weight_max_range = 2,
        },
        {
          influence = 0.75,
          noise_layer = "building-platform",
          noise_octaves_difference = -2.4,
          noise_persistence = 0.3,
          starting_area_weight_optimal = 1,
          starting_area_weight_range = 0,
          starting_area_weight_max_range = 2,
        },
	-- now this tile has higher priority then water. To fix this, we disallow spawn on negative elevation
		{
          influence = -1,
		  max_influence = 0,
          elevation_optimal = -5000,
          elevation_range = 5000,
          elevation_max_range = 5000
        },
	-- adding high frequency noise to modify shape
		{
          influence = -0.25,
		  max_influence = 0,
		  noise_layer = "building-platform",
          noise_octaves_difference = -4,
          noise_persistence = 0.3,
		  starting_area_weight_optimal = 0,
          starting_area_weight_range = 0,
          starting_area_weight_max_range = 2,
        },
      },
};
	
-- making all tiles to block floor-layer
for name,item in pairs(tiles) do
	if not item.collision_mask then item.collision_mask = {} end;
	if item ~= usetile then
		table.insert(item.collision_mask, "floor-layer");
	else
		table.insert(item.collision_mask, "doodad-layer")
	end
end

-- making all craftable tiles to be blocked by doodad-layer
for name,item in pairs(data.raw["item"]) do
	if item.place_as_tile then
		if item.place_as_tile.result == usetile.name then
			item.place_as_tile = nil;
		else
			if not item.place_as_tile.condition then item.place_as_tile.condition = {} end;
			table.insert(item.place_as_tile.condition,"doodad-layer");
		end
	end
end